"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var every = require("lodash/every.js");
var some = require("lodash/some.js");
var startsWith = require("lodash/startsWith.js");
var endsWith = require("lodash/endsWith.js");
var lt = require("lodash/lt.js");
var lte = require("lodash/lte.js");
var gt = require("lodash/gt.js");
var gte = require("lodash/gte.js");
var eq = require("lodash/eq.js");
var map = require("lodash/map.js");
var keyBy = require("lodash/keyBy.js");
var chunk = require("lodash/chunk.js");
var drop = require("lodash/drop.js");
var dropRight = require("lodash/dropRight.js");
var take = require("lodash/take.js");
var takeRight = require("lodash/takeRight.js");
var flattenDepth = require("lodash/flattenDepth.js");
var fromPairs = require("lodash/fromPairs.js");
var nth = require("lodash/nth.js");
var reverse = require("lodash/reverse.js");
var uniq = require("lodash/uniq.js");
var uniqBy = require("lodash/uniqBy.js");
var countBy = require("lodash/countBy.js");
var filter = require("lodash/filter.js");
var reject = require("lodash/reject.js");
var groupBy = require("lodash/groupBy.js");
var sortBy = require("lodash/sortBy.js");
var minBy = require("lodash/minBy.js");
var maxBy = require("lodash/maxBy.js");
var meanBy = require("lodash/meanBy.js");
var sumBy = require("lodash/sumBy.js");
var join = require("lodash/join.js");
var get = require("lodash/get.js");
var set = require("lodash/set.js");
var unset = require("lodash/unset.js");
var assign = require("lodash/assign.js");
var mapValues = require("lodash/mapValues.js");
var at = require("lodash/at.js");
var toPairs = require("lodash/toPairs.js");
var invert = require("lodash/invert.js");
var invertBy = require("lodash/invertBy.js");
var keys = require("lodash/keys.js");
var values = require("lodash/values.js");
var omit = require("lodash/omit.js");
function getType(v) {
    return Object.prototype.toString.call(v).match(/\[object (.+)\]/)[1].toLowerCase();
}
var transformations = {
    array: {
        each: function (array, arg) {
            return map(array, function (item) { return applyTransformations(item, arg); });
        },
        map: map,
        keyBy: keyBy,
        chunk: chunk,
        drop: drop,
        dropRight: dropRight,
        take: take,
        takeRight: takeRight,
        flattenDepth: flattenDepth,
        fromPairs: fromPairs,
        nth: nth,
        reverse: reverse,
        uniq: uniq,
        uniqBy: uniqBy,
        countBy: countBy,
        filter: filter,
        reject: reject,
        filterIf: function (array, arg) {
            return filter(array, function (item) { return applyTransformations(item, arg); });
        },
        rejectIf: function (array, arg) {
            return reject(array, function (item) { return applyTransformations(item, arg); });
        },
        groupBy: groupBy,
        sortBy: sortBy,
        minBy: minBy,
        maxBy: maxBy,
        meanBy: meanBy,
        sumBy: sumBy,
        join: join,
    },
    object: {
        get: get,
        assign: function (src, args) { return (Array.isArray(args) ? args : [args]).reduce(function (obj, path) {
            var source = get(obj, path);
            if (source && typeof source === 'object') {
                return omit(assign(obj, get(obj, path)), path);
            }
            else {
                return obj;
            }
        }, src); },
        mapValues: mapValues,
        at: at,
        toPairs: toPairs,
        invert: invert,
        invertBy: invertBy,
        keys: keys,
        values: values,
    },
    number: {
        lt: lt,
        lte: lte,
        gt: gt,
        gte: gte,
        eq: eq,
    },
    string: {
        startsWith: startsWith,
        endsWith: endsWith,
        match: function (src, args) {
            return src.match(new RegExp(args.match, args.flags));
        },
        isMatch: function (src, args) {
            return (new RegExp(args.match, args.flags)).test(src);
        },
        toJSON: function (str) {
            return JSON.parse(str);
        }
    },
    '*': {
        stringify: function (src) {
            return JSON.stringify(src);
        },
        trim: function (src) {
            if (typeof src === 'string') {
                return src.trim();
            }
            else {
                if (typeof src === 'function') {
                    return function () {
                        var args = [];
                        for (var _i = 0; _i < arguments.length; _i++) {
                            args[_i] = arguments[_i];
                        }
                        var result = src.apply(void 0, args);
                        if (typeof result === 'string') {
                            return result.trim();
                        }
                        else {
                            return result;
                        }
                    };
                }
                else {
                    return src;
                }
            }
        },
        convert: function (obj, type) {
            if (obj !== null || obj !== undefined) {
                switch (type) {
                    case 'toNumber':
                        return parseFloat(obj);
                    case 'toString':
                        return obj.toString();
                    default:
                }
            }
            else {
                switch (type) {
                    case 'toNumber':
                        return NaN;
                    case 'toString':
                        return '';
                    default:
                }
            }
        },
        dive: function (src, args) { return function (obj, key) {
            unset(obj, key);
            set(obj, args, src);
        }; },
    },
};
var opToExpectedType = (function (trans) {
    var result = {};
    for (var type in trans) {
        if (trans.hasOwnProperty(type)) {
            var names = trans[type];
            for (var name in names) {
                if (names.hasOwnProperty(name)) {
                    result[name] = type;
                }
            }
        }
    }
    return result;
})(transformations);
function applyTransformations(object, args) {
    if (!args || object === null || object === undefined) {
        return object;
    }
    for (var op in args) {
        if (args.hasOwnProperty(op)) {
            var arg = args[op];
            if (op === 'and') {
                object = every(arg, function (predicateArgs) { return applyTransformations(object, predicateArgs); });
                continue;
            }
            if (op === 'or') {
                object = some(arg, function (predicateArgs) { return applyTransformations(object, predicateArgs); });
                continue;
            }
            var expectedType = opToExpectedType[op];
            if (!(object === undefined || object === null)) {
                var type = getType(object);
                if (expectedType !== '*' && expectedType !== type && type !== undefined) {
                    debugger;
                    throw Error("\"" + op + "\" transformation expect \"" + expectedType + "\" but got \"" + type + "\"");
                }
            }
            object = transformations[expectedType][op](object, arg);
        }
    }
    return object === undefined ? null : object;
}
exports.applyTransformations = applyTransformations;
//# sourceMappingURL=transformations.js.map